@@ -21,9 +21,9 @@
 #include "file.h"
 
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <sys/stat.h>
 #include <unistd.h>
 #include <utime.h>
 
@@ -111,6 +111,7 @@ ET_Comp_Func_Sort_File_By_Descending_Filename (const ET_File *ETFile1,
 /*
  * Comparison function for sorting by ascending disc number.
  */
+/* FIXME: Handle non-numeric disc number. */
 gint
 et_comp_func_sort_file_by_ascending_disc_number (const ET_File *ETFile1,
                                                  const ET_File *ETFile2)
@@ -161,6 +162,7 @@ et_comp_func_sort_file_by_descending_disc_number (const ET_File *ETFile1,
 /*
  * Comparison function for sorting by ascending track number.
  */
+/* FIXME: Handle non-numeric track number. */
 gint
 ET_Comp_Func_Sort_File_By_Ascending_Track_Number (const ET_File *ETFile1,
                                                   const ET_File *ETFile2)
@@ -255,6 +257,50 @@ ET_Comp_Func_Sort_File_By_Descending_Creation_Date (const ET_File *ETFile1,
     return ET_Comp_Func_Sort_File_By_Ascending_Creation_Date(ETFile2,ETFile1);
 }
 
+/*
+ * et_file_list_sort_string:
+ * @str1: a UTF-8 string, or %NULL
+ * @str2: a UTF-8 string to compare against, or %NULL
+ * @file1: an #ET_File corresponding to @str1
+ * @file2: an #ET_File corresponding to @str2
+ * @case_sensitive: whether the sorting should obey case
+ *
+ * Compare two UTF-8 strings, normalizing them before doing so, falling back to
+ * the filenames if the strings are otherwise identical, and obeying the
+ * requested case-sensitivity.
+ *
+ * Returns: an integer less than, equal to, or greater than zero, if str1 is
+ * less than, equal to or greater than str2
+ */
+static gint
+et_file_list_sort_string (const gchar *str1,
+                          const gchar *str2,
+                          const ET_File *file1,
+                          const ET_File *file2,
+                          gboolean case_sensitive)
+{
+    gint result;
+
+    if (case_sensitive)
+    {
+        result = et_normalized_strcmp0 (str1, str1);
+    }
+    else
+    {
+        result = et_normalized_strcasecmp0 (str1, str2);
+    }
+
+    if (result == 0)
+    {
+        /* Secondary criterion. */
+        return ET_Comp_Func_Sort_File_By_Ascending_Filename (file1, file2);
+    }
+    else
+    {
+        /* Primary criterion. */
+        return result;
+    }
+}
 
 /*
  * Comparison function for sorting by ascending title.
@@ -268,28 +314,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Title (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->title == ((File_Tag *)ETFile2->FileTag->data)->title))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->title )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->title )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->title,((File_Tag *)ETFile2->FileTag->data)->title) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->title,((File_Tag *)ETFile2->FileTag->data)->title);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->title,((File_Tag *)ETFile2->FileTag->data)->title) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-      else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->title,((File_Tag *)ETFile2->FileTag->data)->title);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->title,
+                                     ((File_Tag *)ETFile2->FileTag->data)->title,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -315,28 +354,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Artist (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->artist == ((File_Tag *)ETFile2->FileTag->data)->artist))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->artist )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->artist )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->artist,((File_Tag *)ETFile2->FileTag->data)->artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->artist,((File_Tag *)ETFile2->FileTag->data)->artist);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->artist,((File_Tag *)ETFile2->FileTag->data)->artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->artist,((File_Tag *)ETFile2->FileTag->data)->artist);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->artist,
+                                     ((File_Tag *)ETFile2->FileTag->data)->artist,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -361,28 +393,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Album_Artist (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->album_artist == ((File_Tag *)ETFile2->FileTag->data)->album_artist))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->album_artist )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->album_artist )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->album_artist,((File_Tag *)ETFile2->FileTag->data)->album_artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Artist(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->album_artist,((File_Tag *)ETFile2->FileTag->data)->album_artist);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->album_artist,((File_Tag *)ETFile2->FileTag->data)->album_artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Artist(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->album_artist,((File_Tag *)ETFile2->FileTag->data)->album_artist);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->album_artist,
+                                     ((File_Tag *)ETFile2->FileTag->data)->album_artist,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -407,28 +432,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Album (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->album == ((File_Tag *)ETFile2->FileTag->data)->album))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->album )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->album )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->album,((File_Tag *)ETFile2->FileTag->data)->album) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->album,((File_Tag *)ETFile2->FileTag->data)->album);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->album,((File_Tag *)ETFile2->FileTag->data)->album) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->album,((File_Tag *)ETFile2->FileTag->data)->album);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->album,
+                                     ((File_Tag *)ETFile2->FileTag->data)->album,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -492,26 +510,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Genre (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->genre == ((File_Tag *)ETFile2->FileTag->data)->genre))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->genre ) return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->genre ) return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
+    if (!ETFile1->FileTag->data)
     {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->genre,((File_Tag *)ETFile2->FileTag->data)->genre) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->genre,((File_Tag *)ETFile2->FileTag->data)->genre);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->genre,((File_Tag *)ETFile2->FileTag->data)->genre) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->genre,((File_Tag *)ETFile2->FileTag->data)->genre);
+        return -1;
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->genre,
+                                     ((File_Tag *)ETFile2->FileTag->data)->genre,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -537,28 +550,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Comment (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->comment == ((File_Tag *)ETFile2->FileTag->data)->comment))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->comment )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->comment )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->comment,((File_Tag *)ETFile2->FileTag->data)->comment) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->comment,((File_Tag *)ETFile2->FileTag->data)->comment);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->comment,((File_Tag *)ETFile2->FileTag->data)->comment) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->comment,((File_Tag *)ETFile2->FileTag->data)->comment);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->comment,
+                                     ((File_Tag *)ETFile2->FileTag->data)->comment,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -584,28 +590,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Composer (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->composer == ((File_Tag *)ETFile2->FileTag->data)->composer))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->composer )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->composer )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->composer,((File_Tag *)ETFile2->FileTag->data)->composer) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->composer,((File_Tag *)ETFile2->FileTag->data)->composer);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->composer,((File_Tag *)ETFile2->FileTag->data)->composer) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->composer,((File_Tag *)ETFile2->FileTag->data)->composer);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->composer,
+                                     ((File_Tag *)ETFile2->FileTag->data)->composer,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -631,28 +630,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Orig_Artist (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->orig_artist == ((File_Tag *)ETFile2->FileTag->data)->orig_artist))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->orig_artist )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->orig_artist )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->orig_artist,((File_Tag *)ETFile2->FileTag->data)->orig_artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->orig_artist,((File_Tag *)ETFile2->FileTag->data)->orig_artist);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->orig_artist,((File_Tag *)ETFile2->FileTag->data)->orig_artist) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->orig_artist,((File_Tag *)ETFile2->FileTag->data)->orig_artist);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->orig_artist,
+                                     ((File_Tag *)ETFile2->FileTag->data)->orig_artist,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -678,28 +670,21 @@ ET_Comp_Func_Sort_File_By_Ascending_Copyright (const ET_File *ETFile1,
    ||  (((File_Tag *)ETFile1->FileTag->data)->copyright == ((File_Tag *)ETFile2->FileTag->data)->copyright))
         return 0;
 
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->copyright )
+    if (!ETFile1->FileTag->data)
+    {
         return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->copyright )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->copyright,((File_Tag *)ETFile2->FileTag->data)->copyright) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->copyright,((File_Tag *)ETFile2->FileTag->data)->copyright);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->copyright,((File_Tag *)ETFile2->FileTag->data)->copyright) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->copyright,((File_Tag *)ETFile2->FileTag->data)->copyright);
     }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->copyright,
+                                     ((File_Tag *)ETFile2->FileTag->data)->copyright,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -720,33 +705,29 @@ gint
 ET_Comp_Func_Sort_File_By_Ascending_Url (const ET_File *ETFile1,
                                          const ET_File *ETFile2)
 {
-   // Compare pointers just in case they are the same (e.g. both are NULL)
-   if ((ETFile1->FileTag->data == ETFile2->FileTag->data)
-   ||  (((File_Tag *)ETFile1->FileTag->data)->url == ((File_Tag *)ETFile2->FileTag->data)->url))
+    /* Compare pointers just in case they are the same (e.g. both are NULL). */
+    if ((ETFile1->FileTag->data == ETFile2->FileTag->data)
+        || (((File_Tag *)ETFile1->FileTag->data)->url
+            == ((File_Tag *)ETFile2->FileTag->data)->url))
+    {
         return 0;
-
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->url )
-        return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->url )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->url,((File_Tag *)ETFile2->FileTag->data)->url) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->url,((File_Tag *)ETFile2->FileTag->data)->url);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->url,((File_Tag *)ETFile2->FileTag->data)->url) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->url,((File_Tag *)ETFile2->FileTag->data)->url);
     }
+
+    if (!ETFile1->FileTag->data)
+    {
+        return -1;
+    }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->url,
+                                     ((File_Tag *)ETFile2->FileTag->data)->url,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -767,33 +748,29 @@ gint
 ET_Comp_Func_Sort_File_By_Ascending_Encoded_By (const ET_File *ETFile1,
                                                 const ET_File *ETFile2)
 {
-   // Compare pointers just in case they are the same (e.g. both are NULL)
-   if ((ETFile1->FileTag->data == ETFile2->FileTag->data)
-   ||  (((File_Tag *)ETFile1->FileTag->data)->encoded_by == ((File_Tag *)ETFile2->FileTag->data)->encoded_by))
+    /* Compare pointers just in case they are the same (e.g. both are NULL). */
+    if ((ETFile1->FileTag->data == ETFile2->FileTag->data)
+        || (((File_Tag *)ETFile1->FileTag->data)->encoded_by
+            == ((File_Tag *)ETFile2->FileTag->data)->encoded_by))
+    {
         return 0;
-
-    if ( !ETFile1->FileTag->data || !((File_Tag *)ETFile1->FileTag->data)->encoded_by )
-        return -1;
-    if ( !ETFile2->FileTag->data || !((File_Tag *)ETFile2->FileTag->data)->encoded_by )
-        return 1;
-
-    if (g_settings_get_boolean (MainSettings, "sort-case-sensitive"))
-    {
-        if ( strcmp(((File_Tag *)ETFile1->FileTag->data)->encoded_by,((File_Tag *)ETFile2->FileTag->data)->encoded_by) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcmp(((File_Tag *)ETFile1->FileTag->data)->encoded_by,((File_Tag *)ETFile2->FileTag->data)->encoded_by);
-    }else
-    {
-        if ( strcasecmp(((File_Tag *)ETFile1->FileTag->data)->encoded_by,((File_Tag *)ETFile2->FileTag->data)->encoded_by) == 0 )
-            // Second criterion
-            return ET_Comp_Func_Sort_File_By_Ascending_Filename(ETFile1,ETFile2);
-        else
-            // First criterion
-            return strcasecmp(((File_Tag *)ETFile1->FileTag->data)->encoded_by,((File_Tag *)ETFile2->FileTag->data)->encoded_by);
     }
+
+    if (!ETFile1->FileTag->data)
+    {
+        return -1;
+    }
+
+    if (!ETFile2->FileTag->data)
+    {
+        return 1;
+    }
+
+    return et_file_list_sort_string (((File_Tag *)ETFile1->FileTag->data)->encoded_by,
+                                     ((File_Tag *)ETFile2->FileTag->data)->encoded_by,
+                                     ETFile1, ETFile2,
+                                     g_settings_get_boolean (MainSettings,
+                                                             "sort-case-sensitive"));
 }
 
 /*
@@ -1421,7 +1398,7 @@ ET_Save_File_Tag_To_HD (ET_File *ETFile, GError **error)
                                     "file-update-parent-modification-time"))
         {
             gchar *path = g_path_get_dirname (cur_filename);
-            utime (path, NULL);
+            g_utime (path, NULL);
             g_free (path);
         }
 
@@ -1856,11 +1833,10 @@ ET_File_Format_File_Extension (const ET_File *ETFile)
 
     switch (mode)
     {
-        /* FIXME: Should use filename encoding, not UTF-8! */
         case ET_FILENAME_EXTENSION_LOWER_CASE:
-            return g_utf8_strdown (ETFile->ETFileDescription->Extension, -1);
+            return g_ascii_strdown (ETFile->ETFileDescription->Extension, -1);
         case ET_FILENAME_EXTENSION_UPPER_CASE:
-            return g_utf8_strup (ETFile->ETFileDescription->Extension, -1);
+            return g_ascii_strup (ETFile->ETFileDescription->Extension, -1);
         case ET_FILENAME_EXTENSION_NO_CHANGE:
         default:
             return g_strdup (ETFile->ETFileExtension);
